Ragequit = LibStub("AceAddon-3.0"):NewAddon("Ragequit","AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0")

local HBD = LibStub("HereBeDragons-2.0")
local Pins = LibStub("HereBeDragons-Pins-2.0")

local defaults = {}
local debugMode = false
local officer = false
isOfficerLoaded = false

function Ragequit:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New("RQDB", defaults)

    if (RQDB == nil) then
        RQDB = {}
    end

    local realm = GetRealmName()
    if (RQDB[realm] == nil) then
        RQDB[realm] = {}
    end

    local player = UnitName("player")
    if (RQDB[realm][player] == nil) then
        RQDB[realm][player] = {}
    end

    local faction = UnitFactionGroup("player")
    RQDB[realm][player]["faction"] = faction

    local class = UnitClass("player")
    RQDB[realm][player]["class"] = class
    RQDB[realm][player]["level"] = UnitLevel("player")

    Ragequit:RegisterChatCommand('export', 'HandleExportCommand')
    Ragequit:RegisterEvent("TRADE_SKILL_SHOW")
    Ragequit:RegisterEvent("TRADE_SKILL_UPDATE")
    Ragequit:RegisterEvent("CRAFT_UPDATE")
    Ragequit:RegisterEvent("PLAYER_GUILD_UPDATE")
    Ragequit:RegisterEvent("GUILD_ROSTER_UPDATE")
    if debugMode then
        Ragequit:RegisterEvent("CHAT_MSG_PARTY_LEADER")
        Ragequit:RegisterEvent("CHAT_MSG_PARTY") 
    else
        Ragequit:RegisterEvent("CHAT_MSG_RAID_LEADER")
        Ragequit:RegisterEvent("CHAT_MSG_RAID") 
    end
    -- Ragequit:RegisterEvent("CHAT_MSG_SYSTEM") 
    Ragequit:RegisterEvent("GROUP_ROSTER_UPDATE")

    -- Ragequit:PlayerPosition(f);

    -- C_Timer.NewTicker(60, function()
        -- Ragequit:PlayerPosition(f)
    -- end)
end

function Ragequit:PlayerPosition(f)
    local x, y, instanceID = HBD:GetPlayerWorldPosition()
    local playerName = UnitName("player")
    local className, _, _ = UnitClass("player")

    print(x, y, instanceID)

    Pins:AddMinimapIconWorld("ragequitPlayer" .. playerName, playerIcon, instanceID, x, y, true)
end

function Ragequit:isDebugMode()
    return debugMode
end

function Ragequit:isOfficer()
    return officer
end

function Ragequit:HandleExportCommand(input)
	Ragequit:ExportCharacters()
end

function Ragequit:HandleRagequitCommand(input)
    if input ~= nil then
        if input == "wb" then
            Ragequit:CheckBuffs()
        end
    end
end

function Ragequit:ExportCharacters()
    local dialog = StaticPopup_Show("RQEXPORT")
	local editBox = dialog.editBox
	editBox:SetText(Ragequit:Encode(encode(RQDB)))
	editBox:HighlightText()
	editBox:SetFocus()
end

function Ragequit:GetPlayerInfo()
    local realm = GetRealmName()
    local player = UnitName("player")
    return realm, player
end

function Ragequit:Encode(data)
    local Base64Chars='ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/'

    return ((data:gsub('.', function(x) 
        local r,Base64Chars='',x:byte()
        for i=8,1,-1 do r=r..(Base64Chars%2^i-Base64Chars%2^(i-1)>0 and '1' or '0') end
        return r;
    end)..'0000'):gsub('%d%d%d?%d?%d?%d?', function(x)
        if (#x < 6) then return '' end
        local c=0
        for i=1,6 do c=c+(x:sub(i,i)=='1' and 2^(6-i) or 0) end
        return Base64Chars:sub(c+1,c+1)
    end)..({ '', '==', '=' })[#data%3+1])
end

function Ragequit:Decode(data)
    local Base64Chars='ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/'

    data = string.gsub(data, '[^'..Base64Chars..'=]', '')
    return (data:gsub('.', function(x)
        if (x == '=') then return '' end
        local r,f='',(Base64Chars:find(x)-1)
        for i=6,1,-1 do r=r..(f%2^i-f%2^(i-1)>0 and '1' or '0') end
        return r;
    end):gsub('%d%d%d?%d?%d?%d?%d?%d?', function(x)
        if (#x ~= 8) then return '' end
        local c=0
        for i=1,8 do c=c+(x:sub(i,i)=='1' and 2^(8-i) or 0) end
            return string.char(c)
    end))
end

function Ragequit:InitOfficer(guildRankName)
    if has_value(officerGuildRanks, guildRankName) then

        isOfficer = true
        if isOfficerLoaded == false then

            if Ragequit:isDebugMode() then print("Ragequit:InitOfficer()") end

            isOfficerLoaded = true

            if guildRankName ~= "Class Leader" then

                Ragequit:Hook("ContainerFrameItemButton_OnModifiedClick", true)

                Ragequit:RegisterChatCommand('auction', 'HandleAuctionCommand')
                Ragequit:RegisterChatCommand('offspec', 'HandleAuctionOSCommand')
                Ragequit:RegisterChatCommand('ragequit', 'HandleRagequitCommand')

                local RQRaidButtonInvite = CreateFrame("Button", "rageButtonInvite", RaidFrame, "UIPanelButtonTemplate")
                RQRaidButtonInvite:SetSize(80 ,30)
                RQRaidButtonInvite:SetText("Invite")
                RQRaidButtonInvite:SetPoint("TOPRIGHT", 85, -27)
                RQRaidButtonInvite:SetScript("OnClick", function()
                    StaticPopup_Show("RQINVITE")
                end)

                local RQRaidButtonAttending = CreateFrame("Button", "rageButtonAttending", RaidFrame, "UIPanelButtonTemplate")
                RQRaidButtonAttending:SetSize(80 ,30)
                RQRaidButtonAttending:SetText("Export")
                RQRaidButtonAttending:SetPoint("TOPRIGHT", 85, -62)
                RQRaidButtonAttending:SetScript("OnClick", function()
                    Ragequit:RaidMembers()
                end)
                RQRaidButtonAttending:SetScript("OnUpdate", function()
                    if IsInRaid() then
                        RQRaidButtonAttending:Enable();
                    else
                        RQRaidButtonAttending:Disable();
                    end
                end)

                local RQButtonRoster = CreateFrame("Button", "rageButtonRoster", GuildFrame, "UIPanelButtonTemplate")
                RQButtonRoster:SetSize(80 ,30)
                RQButtonRoster:SetText("Synkroniser")
                RQButtonRoster:SetPoint("TOPRIGHT", 0, 30)
                RQButtonRoster:SetScript("OnClick", function()
                    Ragequit:GuildMembers()
                end)

                local RQRaidButtonBuffs = CreateFrame("Button", "rageButtonBuffs", RaidFrame, "UIPanelButtonTemplate")
                RQRaidButtonBuffs:SetSize(80 ,30)
                RQRaidButtonBuffs:SetText("Buffs")
                RQRaidButtonBuffs:SetPoint("TOPRIGHT", 85, -97)
                RQRaidButtonBuffs:SetScript("OnClick", function()
                    local output = Ragequit:CheckRaidBuffs()
                    local text = splitString(output, "\n")

                    for i = 1, #text do
                        SendChatMessage(text[i], "RAID");
                    end
                end)
                RQRaidButtonBuffs:SetScript("OnEnter", function()
                    GameTooltip:SetOwner(UIParent, "ANCHOR_NONE");
                    GameTooltip:SetPoint("BOTTOMRIGHT", RQRaidButtonBuffs, "TOPLEFT");
                    GameTooltip:ClearLines();
                    GameTooltip:SetText(Ragequit:CheckRaidBuffs());
                end)
                RQRaidButtonBuffs:SetScript("OnLeave", function()
                    GameTooltip:Hide();
                end)
                RQRaidButtonBuffs:SetScript("OnUpdate", function()
                    if IsInRaid() then
                        RQRaidButtonBuffs:Enable();
                    else
                        RQRaidButtonBuffs:Disable();
                    end
                end)
            else
                local RQRaidButtonBuffs = CreateFrame("Button", "rageButtonBuffs", RaidFrame, "UIPanelButtonTemplate")
                RQRaidButtonBuffs:SetSize(80 ,30)
                RQRaidButtonBuffs:SetText("Buffs")
                RQRaidButtonBuffs:SetPoint("TOPRIGHT", 85, -97)
                RQRaidButtonBuffs:SetScript("OnClick", function()
                    local output = Ragequit:CheckRaidBuffs()
                    local text = splitString(output, "\n")

                    for i = 1, #text do
                        SendChatMessage(text[i], "RAID");
                    end
                end)
                RQRaidButtonBuffs:SetScript("OnEnter", function()
                    GameTooltip:SetOwner(UIParent, "ANCHOR_NONE");
                    GameTooltip:SetPoint("BOTTOMRIGHT", RQRaidButtonBuffs, "TOPLEFT");
                    GameTooltip:ClearLines();
                    GameTooltip:SetText(Ragequit:CheckRaidBuffs());
                end)
                RQRaidButtonBuffs:SetScript("OnLeave", function()
                    GameTooltip:Hide();
                end)
                RQRaidButtonBuffs:SetScript("OnUpdate", function()
                    if IsInRaid() then
                        RQRaidButtonBuffs:Enable();
                    else
                        RQRaidButtonBuffs:Disable();
                    end
                end)
            end
        end
    end
end

function Ragequit:TRADE_SKILL_SHOW()
    local realm, player = Ragequit:GetPlayerInfo()

    if (RQDB[realm][player]["tradeskills"] == nil) then
        RQDB[realm][player]["tradeskills"] = {}
    end
end

function Ragequit:PLAYER_GUILD_UPDATE()
    local realm, player = Ragequit:GetPlayerInfo()
    local guildName, guildRankName, guildRankIndex = GetGuildInfo("player")
    RQDB[realm][player]["guildName"] = guildName
    RQDB[realm][player]["guildRankName"] = guildRankName
    RQDB[realm][player]["guildRankIndex"] = guildRankIndex

    Ragequit:InitOfficer(guildRankName)
end

function Ragequit:GUILD_ROSTER_UPDATE()
    local realm, player = Ragequit:GetPlayerInfo()
    local guildName, guildRankName, guildRankIndex = GetGuildInfo("player")
    RQDB[realm][player]["guildName"] = guildName
    RQDB[realm][player]["guildRankName"] = guildRankName
    RQDB[realm][player]["guildRankIndex"] = guildRankIndex

    Ragequit:InitOfficer(guildRankName)
end

function Ragequit:TRADE_SKILL_UPDATE()
    local realm, player = Ragequit:GetPlayerInfo()

    if (RQDB[realm][player]["tradeskills"] == nil) then
        RQDB[realm][player]["tradeskills"] = {}
    end

    local totalRecipes = GetNumTradeSkills()
    local index = GetFirstTradeSkill()

    if (index == 0) then return end

    -- DEFAULT_CHAT_FRAME:AddMessage("RQ: number of skills "..totalRecipies)
    -- DEFAULT_CHAT_FRAME:AddMessage("RQ: first index "..index)

    local tradeskillName, currentLevel, maxLevel = GetTradeSkillLine()
    local data = {}
    data["current"] = currentLevel
    data["max"] = maxLevel
    data["recipes"] = {}

    local name, type
    for i = 1, GetNumTradeSkills() do
        name, cType, _, _, _, _ = GetTradeSkillInfo(i)
        if (name and (cType ~= "header")) then
        	local itemLink = GetTradeSkillItemLink(i)
            table.insert(data["recipes"], {name = name, link = itemLink})
        end
    end

    RQDB[realm][player]["tradeskills"][tradeskillName] = data
end

function Ragequit:CRAFT_UPDATE()
    local craftName = GetCraftName()
    if (craftName == 'Enchanting') then
        local realm, player = Ragequit:GetPlayerInfo()

        if (RQDB[realm][player]["tradeskills"] == nil) then
            RQDB[realm][player]["tradeskills"] = {}
        end

        local totalRecipes = GetNumCrafts()

        if (totalRecipes == 0) then return end

        local name, currentLevel, maxLevel = GetCraftDisplaySkillLine()
        local data = {}
        data["current"] = currentLevel
        data["max"] = maxLevel
        data["recipes"] = {}

        for i = 1, totalRecipes do
            local name, _, cType, _, _, _, _ = GetCraftInfo(i);
            if (name and (cType ~= "header")) then
                local itemLink = GetCraftItemLink(i);
                table.insert(data["recipes"], {name = name, link = itemLink})
            end
        end

        RQDB[realm][player]["tradeskills"]["Enchanting"] = data
    end
end